//(c) 2016 Dmitry Grinberg   http://dmitry.gr   me@dmitry.gr   see LICENSE.txt

#include <util/delay.h>
#include "flash.h"
#include "hw.h"


#define OUR_FLASH_TYPE		0xA289
#define OUR_FLASH_SIZE		0x200000
#define OUR_BLOCK_SIZE		0x010000

void flInit(void)
{
	extHwVpp(false);
	extHwOePin(true);
	extHwWePin(true);
	
	//deselect chip
	extHwCePin(true);
}

void flGetGeometryInfo(uint32_t *flSzP, uint32_t *erzBlkSzP)
{
	if (flSzP)
		*flSzP = OUR_FLASH_SIZE;
	if (erzBlkSzP)
		*erzBlkSzP = OUR_BLOCK_SIZE;
}

static uint8_t readByte(void)
{
	uint8_t ret;
	
	extHwDataPinsMode(false);
	extHwCePin(false);
	extHwOePin(false);
	_delay_us(0.5);
	ret = extHwDataRead();
	extHwOePin(true);
	extHwCePin(true);
	
	return ret;
}

static void writeByte(uint8_t val)
{
	extHwDataPinsMode(true);
	extHwDataWrite(val);
	extHwCePin(false);
	extHwWePin(false);
	_delay_us(0.5);
	extHwWePin(true);
	extHwCePin(true);
}

bool flCheckCompatible(void)
{
	uint16_t intelligentIdentifier;
	uint8_t sta;
	bool ret = false;
	
	extHwLed(true);
	
	extHwSetAddr(0);
	writeByte(0xff);	//read data (reset mode)
	writeByte(0x90);	//read ID
	
	flRead(0, (uint8_t*)&intelligentIdentifier, sizeof(intelligentIdentifier));
	
	if (intelligentIdentifier != OUR_FLASH_TYPE)
		goto out;
	
	writeByte(0xff);	//read data (reset mode)
	writeByte(0x50);	//reset status
	writeByte(0x70);	//read status
	
	ret = !!(readByte() & 0x80);	//we expect this to be set
	
	writeByte(0xff);	//read data (reset mode)
	
out:
	extHwLed(false);
	return ret;
}


bool flRead(uint32_t addr, uint8_t *dst, uint16_t sz)
{
	if (addr >= OUR_FLASH_SIZE)
		return false;
	if (OUR_FLASH_SIZE - addr < sz)
		sz = OUR_FLASH_SIZE - addr;
	
	extHwLed(true);
	
	extHwSetAddr(addr);
	
	while(sz) {
		
		*dst++ = readByte();
		if (--sz)
			extHwIncrAddr();
	}
	
	extHwLed(false);
	
	return true;
}

bool flErase(uint32_t addr)
{
	bool ret = false;
	
	if (addr & (OUR_BLOCK_SIZE - 1))
		return false;
	if (addr >= OUR_FLASH_SIZE)
		return false;
	if (!extHwRyRead())
		return false;
	
	extHwLed(true);
	extHwSetAddr(addr);
	
	writeByte(0x20);
	writeByte(0xd0);
	_delay_us(100);
	while (!extHwRyRead());
	writeByte(0x70);
	ret = (readByte() & 0xa8) == 0x80;
	writeByte(0x50);	//reset status
	writeByte(0xff);
	extHwLed(false);
	
	return ret;
}

bool flWrite(uint32_t addr, const uint8_t *src, uint16_t sz)
{
	bool ret = false;
	
	if (addr >= OUR_FLASH_SIZE)
		return false;
	if (OUR_FLASH_SIZE - addr < sz)
		sz = OUR_FLASH_SIZE - addr;
	if (!extHwRyRead())
		return false;
	
	extHwLed(true);
	extHwSetAddr(addr);
	
	while(sz) {
		writeByte(0x40);
		writeByte(*src++);
		_delay_us(0.1);
		while (!extHwRyRead());
		writeByte(0x70);
		
		if ((readByte() & 0x98) != 0x80)
			goto out;
		writeByte(0x50);	//reset status
		
		if(--sz)
			extHwIncrAddr();
	}
	
	ret = true;
	
out:
	extHwLed(false);
	
	return ret;
}

