//(c) 2016 Dmitry Grinberg   http://dmitry.gr   me@dmitry.gr   see LICENSE.txt

#ifndef _HID_H_
#define _HID_H_


#include <stdbool.h>
#include <stdint.h>
#include "comms.h"
#include "usb.h"



#define HID_EP_SIZE		64		//can be less, if needed (in bytes)
#define HID_EP_SIZE_NAME	EP_SIZE_64	//as a usb.h macro


#define HID_GET_REPORT		0x01
#define HID_GET_IDLE		0x02
#define HID_GET_PROTOCOL	0x03
#define HID_SET_REPORT		0x09
#define HID_SET_IDLE		0x0A
#define HID_SET_PROTOCOL	0x0B


#define HID_DESCR_SZ_DATA	32

////// data report

	#define HID_DATA_OUT_SZ		sizeof(struct CommsPacket)
	#define HID_DATA_IN_SZ		sizeof(struct CommsPacket)

	typedef struct{
		uint8_t data_out[HID_DATA_OUT_SZ];
	}UsbHidReportData;



//for each descriptor please define HID_ENDPT and undef it after (the INTERRUPT IN endpoint the HID will use)
//for each descriptor please define IFACE_NUMBER and undef it afterwards
//for each descriptor please define HID_POLL_INT and undef it afterwards  (poll interval in ms)

#define USB_HID_DESCR_LEN(num_ep)	(9+9+7 * (num_ep))	//length of one of these
#define USB_HID_HID_DESCR_OFFST	9	//offset from one of these to the hid descriptor

#define HID_DESCRIPTOR(sz, num_extra_ep)												\
	/* interface descriptor (follows inline) */									\
		9,			/* sizeof(usbDescrInterface): length of descriptor in bytes */			\
		DESCR_TYP_IFACE,	/* descriptor type */								\
		IFACE_NUMBER,		/* index of this interface */							\
		0,			/* alternate setting for this interface */					\
		1 + num_extra_ep,	/* endpoints: number of endpoint descriptors to follow */			\
		3,			/* interface class (3 = HID) */							\
		0,			/* interface subclass (see HID spec) */						\
		0,			/* interface protocol (see HID spec) */						\
		0,			/* string index for interface */						\
															\
	/* HID descriptor */												\
		9,			/* sizeof(usbDescrHID): length of descriptor in bytes */			\
		DESCR_TYP_HID,		/* descriptor type: HID */							\
		0x11, 0x01,		/* BCD representation of HID version: 1.0 */					\
		0x00,			/* target country code */							\
		0x01,			/* number of HID Report (or other HID class) Descriptor infos to follow */	\
		DESCR_TYP_HID_REPORT,	/* descriptor type: report */							\
		sz, 0,			/* total length of report descriptor */						\
															\
	/*Endpoint descriptor for EP */											\
		7,			/* sizeof(usbDescrEndpoint) */							\
		DESCR_TYP_ENDPT,	/* descriptor type = endpoint */						\
		0x80 | HID_ENDPT,	/* IN endpoint number */							\
		0x03,			/* attrib: Interrupt endpoint */						\
		HID_EP_SIZE, 0,		/* maximum packet size */							\
		HID_POLL_INT,		/* in ms */									\

#define HID_DESCRIPTOR_EXTRA(sz, rx_ep_num)		\
	/*Endpoint descriptor for EP */											\
		7,			/* sizeof(usbDescrEndpoint) */							\
		DESCR_TYP_ENDPT,	/* descriptor type = endpoint */						\
		rx_ep_num,	/* OUT endpoint number*/							\
		0x03,			/* attrib: Interrupt endpoint */						\
		HID_EP_SIZE, 0,		/* maximum packet size */							\
		HID_POLL_INT,		/* in ms */									\


#define HID_DESCRIPTOR_DATA()	HID_DESCRIPTOR(HID_DESCR_SZ_DATA, 0)

uint8_t hidGetDescrData(const uint8_t** descrP);	//return size (pointer is ROM)


#define HID_ENDPT_CONFIG(epNum)						\
	usbEpConfigure(epNum, EP_TYPE_INT | HID_EP_SIZE_NAME | EP_FLAGS_DIR_IN | EP_ENABLED);

#define HID_ENDPT_CONFIG_RX(epNum)						\
	usbEpConfigure(epNum, EP_TYPE_INT | HID_EP_SIZE_NAME | EP_ENABLED);
		


/////report clalback helpers

#define REQ_DESCR_HID_REP_START						\
	else if(descrTyp == DESCR_TYP_HID_REPORT && descrIdx == 0){	\
		dLenH = 0;						\
		if(0){}

#define REQ_DESCR_HID_REP_EACH(iface, func)				\
		else if(wIndex == iface){				\
			dLenL = func(&dPtr);				\
		}

#define REQ_DESCR_HID_REP_END						\
		else return 0;						\
	}



#define REQ_DESCR_HID_DESCR_START					\
	else if(descrTyp == DESCR_TYP_HID && descrIdx == 0){		\
		dLenH = 0;						\
		dLenL = 9;						\
		if(0){}

#define HID_DESCR_HID_DESCR_EACH(iface, ptr)				\
		else if(wIndex == iface){				\
			dPtr = ptr + USB_HID_HID_DESCR_OFFST;		\
		}

#define REQ_DESCR_HID_DESCR_END						\
		else return 0;						\
	}


#define REQ_CUSTOM_EP0_HID_GET_REP_START					\
	else if(BM_REQ_IS_CLS(bmRequestType) && bRequest == HID_GET_REPORT){	\
		if(wIndexH) return false;

#define REQ_CUSTOM_EP0_HID_GET_REP_EACH(iface, ptr, sz_)			\
		else if(wIndexL == iface){					\
			data = ptr;						\
			sz = sz_;						\
		}

#define REQ_CUSTOM_EP0_HID_GET_REP_END						\
		else return false;						\
	}


#define REQ_CUSTOM_EP0_HID_GET_IDLE_START					\
	else if(BM_REQ_IS_CLS(bmRequestType) && bRequest == HID_GET_IDLE){	\
		sz = 1;								\
		if(wIndexH) return false;

#define REQ_CUSTOM_EP0_HID_GET_IDLE_EACH(iface, ptr)				\
		else if(wIndexL == iface){					\
			data = ptr;						\
		}

#define REQ_CUSTOM_EP0_HID_GET_IDLE_END						\
		else return false;						\
	}


#define REQ_CUSTOM_EP0_HID_SET_IDLE_START					\
	else if(BM_REQ_IS_CLS(bmRequestType) && bRequest == HID_SET_IDLE){	\
		sz = 0;								\
		if(wIndexH) return false;

#define REQ_CUSTOM_EP0_HID_SET_IDLE_EACH(iface, ptr)				\
		else if(wIndexL == iface){					\
			*(unsigned char*)(ptr) = wValueH;			\
		}

#define REQ_CUSTOM_EP0_HID_SET_IDLE_END						\
		else return false;						\
	}



#endif
